/* -*- pftp-c -*- */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <inttypes.h>
#include <errno.h>
#include "support.h"
#include "rename_dlg.h"
#include "messagebox.h"
#include "globals.h"
#include <str.h>
#include <pftp_client.h>

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

static int deleteLocalDirectory(const char *path, const char *dirname);

void delete_item()
{
    list_view_file_t *item = NULL;
    size_t c, count = 0;
    pftp_server_t ftp = NULL;
    GtkResponseType ret;
    
    if (curlist) {
	ftp = getListViewFTP(curlist, 0);
	if (!ftp && getListViewName(curlist))
	    return;
	count = selectedListView(curlist, &item);
	if (count == 0) {
	    freeSelectedListView(&item, count);
	    return;
	}
    } else
	return;
        
    if (count == 1) {
	ret = messageBox(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION,
			 GTK_BUTTONS_YES_NO, "Really delete %s?", item[0]);
    } else {
	ret = messageBox(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION,
			 GTK_BUTTONS_YES_NO, "Really delete all %z items?", 
			 count);
    }

    if (ret == GTK_RESPONSE_YES) {
	char *path = NULL;
	if (!pftp_get_path_dir(getListViewName(curlist), &path)) {
	    if (ftp) {
		for (c = 0; c < count; c++) {
		    if (item[c].directory) {
			pftp_deleteDirectoryFTP(ftp, path, item[c].filename);
		    } else {
			pftp_rm(ftp, item[c].filename);
		    }
		    
		    if (pftp_cd(ftp, path, NULL)) {
			break;
		    }
		}

		pftp_clear_dir_cache(ftp, ".");
	    } else {
		for (c = 0; c < count; c++) {
		    if (item[c].directory) {
			if (deleteLocalDirectory(path, item[c].filename)) {
			    chdir(path);
			    break;
			}
		    } else {
			if (unlink(item[c].filename)) {
			    messageBox(NULL, GTK_DIALOG_MODAL, 
				       GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
				       "Error deleting file: `%s'.\n%s",
				       item[c].filename, strerror(errno));
			    chdir(path);
			    break;
			}
		    }
		    
		    if (chdir(path)) {
			break;
		    }
		}
	    }
	}

	if (path) free(path);
	updateListViewDir(curlist);
    }

    freeSelectedListView(&item, count);
}

/*
 * Copy of pftp_deleteDirectoryFTP made for local directories.
 */
int deleteLocalDirectory(const char *path, const char *dirname)
{
    int ret = 0;
    char *fullpath = alloc_strcpy(path);
    if (fullpath[strlen(fullpath) - 1] != '/')
	fullpath = realloc_strcat(fullpath, "/");
    fullpath = realloc_strcat(fullpath, dirname);

    if (!chdir(path) && !chdir(dirname)) {
	pftp_directory_t *ls;
	if ((ls = pftp_trans_ls2(NULL, "*", NULL, 1, 0))) {
	    size_t f;
	    for (f = 0; f < ls->length; f++) {
		if (ls->files[f].type == pft_directory) {
		    if (strcmp(ls->files[f].name, ".") &&
			strcmp(ls->files[f].name, "..")) {
			if (deleteLocalDirectory(fullpath, 
						 ls->files[f].name)) {
			    ret = -1;
			    break;
			}
		    }
		} else {
	            if (unlink(ls->files[f].name)) {
			messageBox(NULL, GTK_DIALOG_MODAL, 
				   GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
				   "Error deleting file: `%s'.\n%s",
				   ls->files[f].name, strerror(errno));
			ret = -1;
			break;
		    }
		}
	    }

	    pftp_free_fdt((*ls));
	    free(ls);
	}
	
	if (!chdir(path)) {	    
	    if (ret != -1 && rmdir(dirname)) {
		messageBox(NULL, GTK_DIALOG_MODAL, 
			   GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
			   "Error deleting directory: `%s'.\n%s",
			   dirname, strerror(errno));
		ret = -1;	    
	    }
	} else {
	    messageBox(NULL, GTK_DIALOG_MODAL, 
		       GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
		       "Error entering directory: `%s'.\n%s",
		       path, strerror(errno));
	    ret = -1;
	}
    } else {
	messageBox(NULL, GTK_DIALOG_MODAL, 
		   GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
		   "Error entering directory: `%s/%s'.\n%s",
		   path, dirname, strerror(errno));
	ret = -1;
    }

    free(fullpath);
    
    return ret;
}
