/* -*- pftp-c -*- */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <inttypes.h>
#include <pftputil.h>
#include <pftp_client.h>
#include <string.h>
#include <unistd.h>
#include <str.h>
#include "list_view.h"
#include "globals.h"

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

struct list_view_s
{
    GtkTreeView *view;
    GtkLabel *label;
    GtkEventBox *label_box;
    GtkListStore *store, *dirstore;
    GtkComboBox *dir;
    int ignore_change, locked;
    char *ftp_name;
    pftp_server_t ftp;
};

static enum {
    ICON_COLUMN,
    FILENAME_COLUMN,
    EXT_COLUMN,
    SIZE_COLUMN,
    N_COLUMNS
} list_view_cols;

static void loadListView(list_view_t view);
static gint filenameCMP(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
			gpointer user_data);
static gint fileExtCMP(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
			gpointer user_data);
static gint fileSizeCMP(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
			gpointer user_data);

static gboolean searchFilenameCMP(GtkTreeModel *model, gint column, 
				  const gchar *key, GtkTreeIter *iter,
				  gpointer search_data);

list_view_t initListView(GtkTreeView *view, GtkLabel *label, 
			 GtkEventBox *label_box, GtkComboBox *dir,
			 const char *color)
{
    GdkColor bg_color;
    GtkCellRenderer *renderer, *iconrenderer;
    GtkTreeViewColumn *column;
    list_view_t ret = malloc(sizeof(struct list_view_s));
    memset(ret, 0, sizeof(struct list_view_s));
    
    ret->view = view;
    ret->label = label;
    ret->label_box = label_box;
    ret->dir = dir;
    
    if (gdk_color_parse(color, &bg_color)) {
	gtk_widget_modify_bg(GTK_WIDGET(ret->label_box), GTK_STATE_SELECTED,
			     &bg_color);
    }
    
    renderer = gtk_cell_renderer_text_new();
    iconrenderer = gtk_cell_renderer_pixbuf_new();

    ret->store = gtk_list_store_new(N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING,
				    G_TYPE_STRING, G_TYPE_STRING);
    gtk_tree_view_set_model(ret->view, GTK_TREE_MODEL(ret->store)); 

    ret->dirstore = gtk_list_store_new(1, G_TYPE_STRING);
    gtk_combo_box_set_model(ret->dir, GTK_TREE_MODEL(ret->dirstore));
    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(ret->dir), renderer, TRUE);
    gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(ret->dir), renderer, 
				  "text", 0);				  
    
    gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(ret->store), 
				    1, filenameCMP, NULL, NULL);
    gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(ret->store), 
				    2, fileExtCMP, NULL, NULL);
    gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(ret->store), 
				    3, fileSizeCMP, NULL, NULL);
    
    column = gtk_tree_view_column_new_with_attributes("", iconrenderer, 
						      "stock-id",
						      ICON_COLUMN, NULL);
    gtk_tree_view_column_set_min_width(column, 32);
    gtk_tree_view_append_column(ret->view, column);
    column = gtk_tree_view_column_new_with_attributes("Filename", renderer, 
						      "text", FILENAME_COLUMN, 
						      NULL);
    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_column_set_sort_column_id(column, 1);
    gtk_tree_view_append_column(ret->view, column);
    column = gtk_tree_view_column_new_with_attributes("Ext", renderer, "text", 
						      EXT_COLUMN, NULL);
    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_column_set_sort_column_id(column, 2);
    gtk_tree_view_append_column(ret->view, column);
    column = gtk_tree_view_column_new_with_attributes("Size", renderer, "text",
						      SIZE_COLUMN, NULL);    
    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_column_set_sort_column_id(column, 3);
    gtk_tree_view_append_column(ret->view, column);

    loadListView(ret);

    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(ret->store), 
					 1, GTK_SORT_ASCENDING);

    gtk_tree_view_set_search_equal_func(ret->view, searchFilenameCMP,
					NULL, NULL);
    gtk_tree_view_set_search_column(ret->view, 1);
    
    return ret;
}

void freeListView(list_view_t view)
{
    if (view) {
	if (view->ftp_name) {
	    pftp_closeFTPCtrl(view->ftp_name);
	    free(view->ftp_name);
	}
	free(view);
    }
}

pftp_server_t getListViewFTP(list_view_t view, int connect)
{
    if (connect) {
	if (!view->ftp) {
	    if (view->ftp_name) {
		return (view->ftp = pftp_getFTPCtrl(view->ftp_name));
	    } else {
		return NULL;
	    }
	} else {
	    return view->ftp;
	}
    } else {
	return view->ftp;
    }
}

void setListViewFTP(list_view_t view, const char *ftp_name)
{
    if (view->ftp_name) {
	pftp_closeFTPCtrl(view->ftp_name);
	view->ftp = NULL;
	free(view->ftp_name);
	view->ftp_name = NULL;
    } 
    
    if (ftp_name) {
	view->ftp_name = (char *) strdup(ftp_name);
    } else {
	view->ftp_name = NULL;
    }
    
    loadListView(view);
}

void loadListView(list_view_t view)
{
    if (view->ftp_name) {
	if (view->ftp_name[0] == '\n')
	    gtk_label_set_text(view->label, view->ftp_name + 1);
	else
	    gtk_label_set_text(view->label, view->ftp_name);
    } else {  
	gtk_label_set_text(view->label, "[local]");
    }
    
    updateListViewDir(view);
}

void refreshListView(list_view_t view)
{
    if (view->ftp_name) {
	pftp_server_t ftp = getListViewFTP(view, 0);
	
	if (ftp) 
	    pftp_clear_dir_cache(ftp, ".");
    }
    
    updateListViewDir(view);
}

static void fillDirStore(list_view_t view, const char *path)
{
    size_t pathlen = strlen(path);
    char *pos, *tmp, *last;
    GtkTreeIter iter;

    view->ignore_change = 1;

    gtk_list_store_clear(view->dirstore);

    tmp = malloc(pathlen + 3);
    memcpy(tmp, path, pathlen);
    if (tmp[pathlen - 1] == '/')
	tmp[pathlen] = '\0';
    else
	memcpy(tmp + pathlen, "/", 2);

    last = tmp;

    while ((pos = strchr(last, '/'))) {
	char old = pos[1];
	pos[1] = '\0';
	gtk_list_store_prepend(view->dirstore, &iter);
	gtk_list_store_set(view->dirstore, &iter, 0, tmp, -1);
	pos[1] = old;
	last = pos + 1;
    }

    gtk_combo_box_set_active(view->dir, 0);

    free(tmp);

    view->ignore_change = 0;
}

void updateListViewDir(list_view_t view)
{
    pftp_directory_t *dir;
    char *cur = NULL;
    int is_root = 0;
    GtkTreeIter iter;
    
    gtk_list_store_clear(view->store);
    
    if (!pftp_get_path_dir(view->ftp_name, &cur)) {
	if (strcmp(cur, "/") == 0)
	    is_root = 1;
    }

    fillDirStore(view, cur);
    
    if (cur)
	free(cur);
    
    dir = pftp_trans_ls(view->ftp_name, "*", NULL, 1, 1);
    
    if (!is_root) {
	gtk_list_store_append(view->store, &iter);
	gtk_list_store_set(view->store, &iter, ICON_COLUMN, "gtk-undo",
			   FILENAME_COLUMN, "..", EXT_COLUMN, "<dir>", 
			   SIZE_COLUMN, "", -1);
    }

    if (dir) {
	size_t f, len;
	char *size = malloc(10), *filename = NULL, *ext = NULL, *pos;    
	
	for (f = 0; f < dir->length; f++) {
	    len = strlen(dir->files[f].name);
	    gtk_list_store_append(view->store, &iter);
    
	    if (dir->files[f].type == pft_directory) {
		/* Only glftpd that does this or? */
		/*	
		  if (dir->files[f].size < 1000) {
		  snprintf(size, 10, "%llu KB", dir->files[f].size);
		  } else if (dir->files[f].size < 1000000ULL) {
		  snprintf(size, 10, "%.2f MB", (float)dir->files[f].size / 1000.0f);
		  } else {
		  snprintf(size, 10, "%.2f GB", 
		  (float)dir->files[f].size / 1000000.0f);
		  } */
		filename = realloc(filename, len + 1);
		memcpy(filename, dir->files[f].name, len + 1);
		if ((pos = strstr(filename, " -> "))) pos[0] = '\0';
		gtk_list_store_set(view->store, &iter, ICON_COLUMN, 
				   "gtk-directory", FILENAME_COLUMN, filename, 
				   EXT_COLUMN,  "<dir>", SIZE_COLUMN, "", -1);
	    } else {
		if (dir->files[f].size < 1000) {
		    snprintf(size, 10, "%llu B", dir->files[f].size);
		} else if (dir->files[f].size < 1000000ULL) {
		    snprintf(size, 10, "%.2f KB", 
			     (float)dir->files[f].size / 1000.0f);
		} else if (dir->files[f].size < 1000000000ULL) {
		    snprintf(size, 10, "%.2f MB", 
			     (float)dir->files[f].size / 1000000.0f);
		} else {
		    snprintf(size, 10, "%.2f GB", 
			     (float)dir->files[f].size / 1000000000.0f);
		}
		filename = realloc(filename, len + 1);
		memcpy(filename, dir->files[f].name, len + 1);
		if ((pos = strstr(filename, " -> "))) pos[0] = '\0';
		pos = strrchr(filename, '.');
		if (pos && pos > filename) {
		    len = strlen(pos);
		    pos[0] = '\0';
		    pos++;
		    ext = realloc(ext, len);
		    memcpy(ext, pos, len);
		} else {
		    if (!ext) ext = malloc(1);
		    ext[0] = '\0';
		}
		gtk_list_store_set(view->store, &iter, ICON_COLUMN, "gtk-file",
				   FILENAME_COLUMN, filename, EXT_COLUMN, 
				   ext, SIZE_COLUMN, size, -1);
	    }
	}
	
	free(size);
	if (ext) free(ext);
	if (filename) free(filename);

	pftp_free_fdt((*dir));
	free(dir);
    }

    gtk_tree_view_columns_autosize(view->view);
}

void selectListView(list_view_t view, int select)
{
    if (select) {
	curlist = view;
	gtk_widget_set_state(GTK_WIDGET(view->label), GTK_STATE_SELECTED);
	gtk_widget_set_state(GTK_WIDGET(view->label_box), GTK_STATE_SELECTED);
    } else {
	gtk_widget_set_state(GTK_WIDGET(view->label), GTK_STATE_NORMAL);
	gtk_widget_set_state(GTK_WIDGET(view->label_box), GTK_STATE_NORMAL);
	gtk_widget_set_sensitive(GTK_WIDGET(view->label), TRUE);
    }
}

const char *getListViewName(list_view_t view)
{
    return view->ftp_name;
}

gboolean searchFilenameCMP(GtkTreeModel *model, gint column, 
			   const gchar *key, GtkTreeIter *iter,
			   gpointer search_data)
{
    GValue value;
    int ret;
    memset(&value, 0, sizeof(GValue));

    gtk_tree_model_get_value(model, iter, column, &value);    

    fprintf(stderr, "key: `%s'\n", key);

    ret = strncasecmp(g_value_get_string(&value), key, strlen(key));

    g_value_unset(&value);    

    return (ret != 0);
}

static gint simpeFilenameCMP(GtkTreeModel *model, GtkTreeIter *a, 
			     GtkTreeIter *b, gpointer user_data)
{
    GValue a_value, b_value;
    int ret;

    memset(&a_value, 0, sizeof(GValue));
    memset(&b_value, 0, sizeof(GValue));
    
    gtk_tree_model_get_value(model, a, FILENAME_COLUMN, &a_value);
    gtk_tree_model_get_value(model, b, FILENAME_COLUMN, &b_value);
    
    ret = strcmp(g_value_get_string(&a_value), g_value_get_string(&b_value));
    
    g_value_unset(&a_value);
    g_value_unset(&b_value);
    
    return ret;
}

gint filenameCMP(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
		 gpointer user_data)
{
    GValue a_value, b_value;
    const char *a_ext, *b_ext;
    
    memset(&a_value, 0, sizeof(GValue));
    memset(&b_value, 0, sizeof(GValue));
    
    gtk_tree_model_get_value(model, a, EXT_COLUMN, &a_value);
    gtk_tree_model_get_value(model, b, EXT_COLUMN, &b_value);
    
    a_ext = g_value_get_string(&a_value);
    b_ext = g_value_get_string(&b_value);
    
    if (!strcmp(a_ext, "<dir>") && strcmp(b_ext, "<dir>")) {
	g_value_unset(&a_value);
	g_value_unset(&b_value);
	return -1;
    }
    if (strcmp(a_ext, "<dir>") && !strcmp(b_ext, "<dir>")) {
	g_value_unset(&a_value);
	g_value_unset(&b_value);
	return 1;
    }
    
    g_value_unset(&a_value);
    g_value_unset(&b_value);

    return simpeFilenameCMP(model, a, b, user_data);
}

gint fileExtCMP(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
		gpointer user_data)
{
    GValue a_value, b_value;
    int ret;
    const char *a_ext, *b_ext;
    
    memset(&a_value, 0, sizeof(GValue));
    memset(&b_value, 0, sizeof(GValue));
    
    gtk_tree_model_get_value(model, a, EXT_COLUMN, &a_value);
    gtk_tree_model_get_value(model, b, EXT_COLUMN, &b_value);
    
    a_ext = g_value_get_string(&a_value);
    b_ext = g_value_get_string(&b_value);

    if (!strcmp(a_ext, "<dir>")) {
	if (!strcmp(b_ext, "<dir>"))
	    return simpeFilenameCMP(model, a, b, user_data);

	return -1;
    }
    if (!strcmp(b_ext, "<dir>")) {
	return 1;
    }
    
    ret = strcmp(g_value_get_string(&a_value), g_value_get_string(&b_value));
    
    g_value_unset(&a_value);
    g_value_unset(&b_value);

    if (ret == 0)
	return simpeFilenameCMP(model, a, b, user_data);
    else
	return ret;
}

static uint64_t getFileSize(const char *size)
{
    size_t len = strlen(size);
    uint64_t s = 0;

    if (len == 0)
	return 0;

    if (is_whitespace(size[len - 2])) {
	if (sscanf(size, "%llu B", &s) != 1)
	    return 0;
    } else {
	char c = 0;
	float f = 0.0f;
	
	if (sscanf(size, "%f %cB", &f, &c) != 2)
	    return 0;

	switch (c) {
	case 'K':
	    s = (uint64_t) (f * 1000.0f);
	    break;
	case 'M':
	    s = (uint64_t) (f * 1000000.0f);
	    break;
	case 'G':
	    s = (uint64_t) (f * 1000000000.0f);
	    break;
	}
    }

    return s;
}

gint fileSizeCMP(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
		 gpointer user_data)
{
    GValue a_value, b_value;
    uint64_t a_size, b_size;
    const char *a_ext, *b_ext;
    
    memset(&a_value, 0, sizeof(GValue));
    memset(&b_value, 0, sizeof(GValue));
    
    gtk_tree_model_get_value(model, a, EXT_COLUMN, &a_value);
    gtk_tree_model_get_value(model, b, EXT_COLUMN, &b_value);
    
    a_ext = g_value_get_string(&a_value);
    b_ext = g_value_get_string(&b_value);
    
    if (!strcmp(a_ext, "<dir>") && strcmp(b_ext, "<dir>")) {
	g_value_unset(&a_value);
	g_value_unset(&b_value);
	return -1;
    }
    if (strcmp(a_ext, "<dir>") && !strcmp(b_ext, "<dir>")) {
	g_value_unset(&a_value);
	g_value_unset(&b_value);
	return 1;
    }
   
    g_value_unset(&a_value);
    g_value_unset(&b_value);
    
    gtk_tree_model_get_value(model, a, SIZE_COLUMN, &a_value);
    gtk_tree_model_get_value(model, b, SIZE_COLUMN, &b_value);
    
    a_ext = g_value_get_string(&a_value);
    b_ext = g_value_get_string(&b_value);
    
    if (!a_ext[0]) {
	if (!b_ext[0])
	    return simpeFilenameCMP(model, a, b, user_data);

	return -1;
    }
    if (!b_ext[0]) {
	return 1;
    }

    a_size = getFileSize(a_ext);
    b_size = getFileSize(b_ext);

    g_value_unset(&a_value);
    g_value_unset(&b_value);

    if (a_size < b_size)
	return -1;
    if (a_size > b_size)
	return 1;

    return simpeFilenameCMP(model, a, b, user_data);
}

void actionRowListView(list_view_t view, GtkTreePath *path)
{
    GtkTreeIter pos;
    
    if (gtk_tree_model_get_iter(GTK_TREE_MODEL(view->store), &pos, path)) {
	GValue value;
    
	memset(&value, 0, sizeof(GValue));
	
	gtk_tree_model_get_value(GTK_TREE_MODEL(view->store), &pos, EXT_COLUMN,
				 &value);
	
	if (strcmp(g_value_get_string(&value), "<dir>") == 0) {
	    int err = -1;
	    g_value_unset(&value);
	    gtk_tree_model_get_value(GTK_TREE_MODEL(view->store), &pos, 
				     FILENAME_COLUMN, &value);
	    
	    if (view->ftp_name) {
		pftp_server_t ftp = getListViewFTP(view, 1);
		
		if (ftp)
		    err = pftp_cd(ftp, g_value_get_string(&value), NULL);
	    } else {
		err = chdir(g_value_get_string(&value));
	    }
	    
	    if (!err)
		updateListViewDir(view);
	} else {
	    list_view_file_t *files = malloc(1 * sizeof(list_view_file_t));
	    size_t len;
	    g_value_unset(&value);
	    gtk_tree_model_get_value(GTK_TREE_MODEL(view->store), &pos, 
				     FILENAME_COLUMN, &value);
	    files[0].filename = strdup(g_value_get_string(&value));
	    files[0].directory = 0;
	    g_value_unset(&value);
	    gtk_tree_model_get_value(GTK_TREE_MODEL(view->store), &pos, 
				     EXT_COLUMN, &value);
	    if ((len = strlen(g_value_get_string(&value))) > 0) {
		files[0].filename = realloc(files[0].filename, 
					    strlen(files[0].filename) 
					    + len + 3);
		strcat(files[0].filename, ".");
		strcat(files[0].filename, g_value_get_string(&value));
	    }
	    enqueueView(queue, view, view == left ? right : left, 1, files);
	    free(files[0].filename);
	    free(files);
	}
	
	g_value_unset(&value);
    }
}

void freeSelectedListView(list_view_file_t **item, size_t count)
{
    if (item && (*item)) {
	size_t i;
	for (i = 0; i < count; i++)
	    free((*item)[i].filename);

	free((*item));
	(*item) = NULL;
    }
}

size_t selectedListView(list_view_t view, list_view_file_t **item)
{
    GtkTreeSelection *sel = gtk_tree_view_get_selection(view->view);
    GList *list, *cur;
    int cnt = gtk_tree_selection_count_selected_rows(sel), c;
    GValue value;
    size_t len;
    
    (*item) = realloc((*item), cnt * sizeof(list_view_file_t));
    
    list = gtk_tree_selection_get_selected_rows(sel, NULL);
    memset(&value, 0, sizeof(GValue));
    
    cur = list;
    for (c = 0; c < cnt; c++) {
	GtkTreeIter iter;
	gtk_tree_model_get_iter(GTK_TREE_MODEL(view->store), &iter, 
				(GtkTreePath *) cur->data);
	gtk_tree_model_get_value(GTK_TREE_MODEL(view->store), &iter, 
				 FILENAME_COLUMN, &value); 
	cur = cur->next;
	(*item)[c].filename = strdup(g_value_get_string(&value));
	g_value_unset(&value);
	gtk_tree_model_get_value(GTK_TREE_MODEL(view->store), &iter, 
				 EXT_COLUMN, &value); 
	if ((len = strlen(g_value_get_string(&value))) > 0) {
	    if (strcmp(g_value_get_string(&value), "<dir>") != 0) {
		(*item)[c].filename = realloc((*item)[c].filename, 
					      strlen((*item)[c].filename) 
					      + len + 3);
		strcat((*item)[c].filename, ".");
		strcat((*item)[c].filename, g_value_get_string(&value));
		(*item)[c].directory = 0;
	    } else {
		(*item)[c].directory = 1;
	    }
	} else {
	    (*item)[c].directory = 0;
	}
	
	g_value_unset(&value);
    }
    
    g_list_foreach(list, (GFunc) gtk_tree_path_free, NULL);
    g_list_free(list);  
    
    return cnt;
}

void changeDirectoryListView(list_view_t view)
{
    int err = 1;
    GtkTreeIter pos;
    GValue value;
    memset(&value, 0, sizeof(GValue));

    if (!view || view->ignore_change)
	return;
    
    if (!gtk_combo_box_get_active_iter(view->dir, &pos))
	return;
    
    selectListView(view, 1);
    selectListView(view == left ? right : left, 0);
    
    gtk_tree_model_get_value(GTK_TREE_MODEL(view->dirstore), &pos, 0, &value);
    
    if (view->ftp_name) {
	pftp_server_t ftp = getListViewFTP(view, 1);

	if (ftp)
	    err = pftp_cd(ftp, g_value_get_string(&value), NULL);
    } else {
	err = chdir(g_value_get_string(&value));
    }
    
    if (!err)
	updateListViewDir(view);
    
    g_value_unset(&value);   
}

void lockListViews(void)
{
    lockListView(left);
    lockListView(right);
}

void lockListView(list_view_t view)
{
    view->locked++;
}

void unlockListViews(void)
{
    unlockListView(left);
    unlockListView(right);
}

void unlockListView(list_view_t view)
{
    view->locked--;
}

int lockedListView(list_view_t view)
{
    return (view->locked > 0);
}
