/* -*- pftp-c -*- */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <inttypes.h>
#include <pftputil.h>
#include <stdlib.h>
#include <string.h>
#include "load_menu.h"
#include "callbacks.h"

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

struct load_menu_s {
    GtkMenu *menu;
};

load_menu_t initLoadMenu(GtkMenu *menu)
{
    load_menu_t ret = (load_menu_t) malloc(sizeof(struct load_menu_s));
    memset(ret, 0, sizeof(struct load_menu_s));
    
    ret->menu = menu;
    
    return ret;
}

void freeLoadMenu(load_menu_t menu)
{
    if (menu) {
	free(menu);
    }
}

static void updateMenuItem(GtkWidget *widget, gpointer data);

static char *getLoadMenuItemName(size_t site);

void updateLoadMenu(load_menu_t menu)
{
    size_t cur = 0;
    
    gtk_container_foreach(GTK_CONTAINER(menu->menu), updateMenuItem, &cur);
    
    if (pftp_getNrOfSettings()) {
	while (cur < pftp_getNrOfSettings()) {
	    char *name = getLoadMenuItemName(cur);  
	    GtkWidget *item = gtk_menu_item_new_with_label(name);
	    gtk_widget_show(item);
	    gtk_menu_append(GTK_MENU_SHELL(menu->menu), item);
	    g_signal_connect(G_OBJECT(item), "activate",
			     G_CALLBACK(loadmenuitem_activate), 
			     (gpointer) cur);
	    free(name);
	    cur++;
	}
    } else {
	GtkWidget *item = gtk_menu_item_new_with_label("No sites to load from");
	gtk_widget_show(item);
	gtk_menu_append(GTK_MENU_SHELL(menu->menu), item);
    }
}

static void updateMenuItem(GtkWidget *widget, gpointer data)
{
    size_t *cur = (size_t *)data;
    
    if ((*cur) < pftp_getNrOfSettings()) {
	char *name = getLoadMenuItemName((*cur));    
	gtk_label_set_text(GTK_LABEL(gtk_bin_get_child(GTK_BIN(widget))), 
			   name);
	free(name);
	if ((*cur) == 0) {
	    g_signal_connect(G_OBJECT(widget), "activate",
			     G_CALLBACK(loadmenuitem_activate), 
			     (gpointer) (*cur));
	}
    } else {
	gtk_widget_destroy(widget);
    }
    
    (*cur)++;
}

char *getLoadMenuItemName(size_t site)
{
    pftp_settings_t settings = pftp_getFTPSettings2(site);
    if (settings) {
	return (char *) strdup(settings->name);
    } else {
	return (char *) strdup("<err>");
    } 
}
