/* -*- pftp-c -*- */

/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <gtk/gtk.h>
#include <inttypes.h>
#include <stdio.h>
#include <pftputil.h>
#include <string.h>

#include "interface.h"
#include "support.h"
#include "list_view.h"
#include "queue_view.h"
#include "output_view.h"
#include "load_menu.h"
#include "need_pass_dlg.h"
#include "statusbar.h"
#include "overwrite_dlg.h"
#include "rawcmd_dlg.h"
#include "newdir_dlg.h"
#include "rename_dlg.h"
#include "hpanes.h"

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

list_view_t left, right, curlist = NULL;
output_view_t output;
queue_view_t queue;  
load_menu_t loadmenu;
need_pass_dlg_t needpassdlg;
rawcmd_dlg_t rawcmddlg;
newdir_dlg_t newdirdlg;
rename_dlg_t renamedlg;
statusbar_t statusbar;
static int quit = 0;
int abort_que = 0, abort_transfer = 0;
pftp_conf_t global_config;
GtkWidget *main_window;  

static int comm_central(pftp_msg_t msg, pftp_server_t ftp, void *userdata,
			pftp_param_t param1, pftp_param_t param2);

int main (int argc, char *argv[])
{

#ifdef ENABLE_NLS
    bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
    textdomain (GETTEXT_PACKAGE);
#endif

    gtk_set_locale();
    gtk_init(&argc, &argv);
    
    add_pixmap_directory(PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");
    
    main_window = create_main_window();
    
    left = initListView(GTK_TREE_VIEW(lookup_widget(main_window, "filelist1")),
			GTK_LABEL(lookup_widget(main_window, "view_label1")),
			GTK_EVENT_BOX(lookup_widget(main_window, 
						    "view_label1_box")),
			GTK_COMBO_BOX(lookup_widget(main_window, 
						    "directory1")),
			"darkgreen");
    right = initListView(GTK_TREE_VIEW(lookup_widget(main_window, 
						     "filelist2")),
			 GTK_LABEL(lookup_widget(main_window, "view_label2")),
			 GTK_EVENT_BOX(lookup_widget(main_window, 
						     "view_label2_box")),
			 GTK_COMBO_BOX(lookup_widget(main_window, 
						     "directory2")),
			 "darkblue");
    queue = initQueueView(GTK_TREE_VIEW(lookup_widget(main_window, "queue")));
    output = initOutputView(GTK_TEXT_VIEW(lookup_widget(main_window, 
							"ftpoutput")));
    loadmenu = initLoadMenu(GTK_MENU(lookup_widget(main_window, "load_menu")));
    statusbar = initStatusBar(GTK_LABEL(lookup_widget(main_window, 
						      "statusbar_text")), 
			      GTK_PROGRESS_BAR(lookup_widget(main_window, 
							     "statusbar_progress1")), 
			      GTK_LABEL(lookup_widget(main_window, 
						      "statusbar_text1")), 
			      GTK_PROGRESS_BAR(lookup_widget(main_window, 
							     "statusbar_progress2")), 
			      GTK_LABEL(lookup_widget(main_window, 
						      "statusbar_text2")));
    
    needpassdlg = initNeedPassDlg(create_need_pass_dlg(), main_window);
    initOverwriteDlg(GTK_DIALOG(create_overwrite_dlg()), main_window);
    rawcmddlg = initRawCmdDlg(create_rawcmd_dlg(), main_window);
    newdirdlg = initNewDirDlg(create_newdir_dlg(), main_window);
    renamedlg = initRenameDlg(create_rename_dlg(), main_window);
    
    selectListView(left, 1);
    selectListView(right, 0);

    pftp_initUtil(comm_central);
    
    global_config = pftp_getConfig();
    if (global_config->gui_width == 0)
	global_config->gui_width = 640;
    if (global_config->gui_height == 0)
	global_config->gui_height = 480;
    
    setHPanes(main_window, global_config->horzpan1, global_config->horzpan2);
  
    gtk_window_set_default_size(GTK_WINDOW(main_window), 
				global_config->gui_width, 
				global_config->gui_height);
  
    updateLoadMenu(loadmenu);
  
    gtk_widget_show(main_window);

    output_printf(output, STATUS, "PFTP started...\n");
  
    gtk_main();
    
    quit = 1;

    pftp_saveFTPSettings(NULL, NULL);
    pftp_freeUtil();
    
    freeListView(left);
    freeListView(right);
    freeOutputView(output);
    freeQueueView(queue);
    freeNeedPassDlg(needpassdlg);
    freeRawCmdDlg(rawcmddlg);
    freeNewDirDlg(newdirdlg);
    freeRenameDlg(renamedlg);
    freeStatusBar(statusbar);

    return 0;
}

int comm_central(pftp_msg_t msg, pftp_server_t ftp, void *userdata, 
		 pftp_param_t param1, pftp_param_t param2)
{
  static size_t que_pos = 0, que_len = 0;
  static int file_transfer = 0;
  static uint64_t file_pos = 0, file_len = 0, file_start = 0;
  
  if (quit) {
      /* So that no output and such are tried to get written. */
      return 0;
  }
  
  switch (msg) {
  case PFTP_STATUS: {
      output_type_t type;
      const char *left_name = getListViewName(left);
      const char *right_name = getListViewName(right);
      
      if (left_name && strcmp((const char *)param1, left_name) == 0) {
	  type = LIST1;
      } else if (right_name && strcmp((const char *)param1, right_name) == 0) {
	  type = LIST2;
      } else {
	  type = STATUS;
	  output_printf(output, type, "%s: ", (const char *)param1);
      }
      
      output_printf(output, type, (const char *)param2);
      output_printf(output, type, "\n");
      while (gtk_events_pending())
	  gtk_main_iteration();
      return 0;
  };
  case PFTP_NEEDPASS: {
      return doNeedPassDlg(needpassdlg, (char **)param1, (char **)param2);
  };
  case PFTP_INITTRANSFER:
      lockListViews();
      abort_transfer = 0;
      if (file_transfer) {
	  if (file_len > 0) {
	      updateStatusBar(statusbar, NULL, (file_pos * 100) / file_len, 
			      "File", -1, NULL);
	  } else {
	      updateStatusBar(statusbar, NULL, 666, "File", -1, NULL);
	  }
      } else {
	  updateStatusBar(statusbar, NULL, 666, "Waiting", -1, NULL);
      }
      return 0;
  case PFTP_FILETRANSFER:
      file_transfer = 1;
      file_start = (*(const uint64_t *)param1);
      file_len = (*(const uint64_t *)param2);
      return 0;
  case PFTP_TRANSFER:
      if (file_transfer) {
	  char speed[30];
	  if (param1)
	      snprintf(speed, 30, "File: [%.2fKB/s]", 
		       (*(double *)param1) / 1024.0);
	  else
	      strcpy(speed, "File: [--]");
	  if (file_len > 0) {
	      file_pos = ((float)(file_len - file_start) * 
			  ((*(const float *)param2) / 100.0f)) + file_start;
	      updateStatusBar(statusbar, NULL, (file_pos * 100) / file_len, 
			      speed, -1, NULL);
	  } else {
	      updateStatusBar(statusbar, NULL, 666, speed, -1, NULL);
	  }
      } else {
	  char speed[30];
	  if (param1)
	      snprintf(speed, 30, "Listing: [%.2fKB/s]", 
		       (*(double *)param1) / 1024.0);
	  else
	      strcpy(speed, "Waiting...");
	  updateStatusBar(statusbar, NULL, 666, speed, -1, NULL);
      }
      while (gtk_events_pending())
	  gtk_main_iteration();
      return abort_transfer ? 0 : 1;
  case PFTP_DONETRANSFER:
      updateStatusBar(statusbar, NULL, 0, "", -1, NULL);
      file_transfer = 0;
      unlockListViews();
      return 0;
  case PFTPUTIL_INITQUE: {
      pftp_server_t ftp;
      lockListViews();
      que_pos = 0;
      abort_que = 0;
      que_len = (*(size_t *)param1);
      if (que_len > 0) {
	  updateStatusBar(statusbar, NULL, -1, NULL, (que_pos * 100) / que_len,
			  "Que");
      } else {
	  updateStatusBar(statusbar, NULL, -1, NULL, 0, "Que");
      }
      if ((ftp = getQueFtpTarget(queue))) {
	  pftp_clear_dir_cache(ftp, ".");
      }
      return 0;
  }; break;
  case PFTPUTIL_UPDATEQUE:
      updateQueueView(queue);
      que_len = (*(size_t *)param1) + que_pos;
      if (que_len > 0) {
	  updateStatusBar(statusbar, NULL, -1, NULL, (que_pos * 100) / que_len,
			  "Que");    
      } else {
	  updateStatusBar(statusbar, NULL, -1, NULL, que_len, "Que");    
      }
      return 0;
  case PFTPUTIL_QUE: {
      que_pos = (*(size_t *)param1) + (*(size_t *)param2);
      if (que_len > 0) {
	  updateStatusBar(statusbar, NULL, -1, NULL, (que_pos * 100) / que_len,
			  "Que");
      } else {
	  updateStatusBar(statusbar, NULL, -1, NULL, 0, "Que");
      }
      while (gtk_events_pending())
	  gtk_main_iteration();
      updateQueueView(queue);
      return abort_que ? 0 : 1;
  }; break;
  case PFTPUTIL_DONEQUE:
      updateQueueView(queue);
      updateStatusBar(statusbar, NULL, -1, NULL, 0, "");
      refreshListView(left);
      refreshListView(right);
      unlockListViews();
      return 0;
  case PFTPUTIL_NEEDCURFTP: {
      const char *ftp_name = (const char *)param1;
      pftp_server_t *ret = (pftp_server_t *)param2;

      (*ret) = NULL;
      
      if (curlist && ftp_name && getListViewName(curlist) &&
	  strcmp(getListViewName(curlist), ftp_name) == 0) {
	  (*ret) = getListViewFTP(curlist, 1);
      } else {
	  (*ret) = pftp_getFTPCtrl(ftp_name);
      }
      
      return (*ret) ? 0 : -1;
  }; break;
  case PFTPUTIL_ERROR:
      output_printf(output, ERR, (const char *)param1);
      output_printf(output, ERR, "\n");
      return 0;
  }

  return 0;
}
