/* -*- pftp-c -*- */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <inttypes.h>
#include <pftputil.h>
#include <sys/stat.h>
#include <errno.h>
#include "support.h"
#include "newdir_dlg.h"
#include "messagebox.h"
#include "globals.h"
#include "str.h"

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

// From main.c
extern newdir_dlg_t newdirdlg;

struct newdir_dlg_s {
    GtkDialog *dlg;
    GtkEntry *dirname;
};

newdir_dlg_t initNewDirDlg(GtkWidget *dlg, GtkWidget *parent)
{
    newdir_dlg_t ret = malloc(sizeof(struct newdir_dlg_s));
    memset(ret, 0, sizeof(struct newdir_dlg_s));
    
    ret->dlg = GTK_DIALOG(dlg);
    
    gtk_window_set_transient_for(GTK_WINDOW(ret->dlg), GTK_WINDOW(parent));
    gtk_dialog_set_default_response(ret->dlg, GTK_RESPONSE_OK);
    
    ret->dirname = GTK_ENTRY(lookup_widget(dlg, "new_dir_name"));
    
    return ret;
}

void freeNewDirDlg(newdir_dlg_t dlg)
{
    if (dlg) {
	free(dlg);
    }
}

static int doNewDirDlg(newdir_dlg_t dlg, char **dir)
{
    int ret;
    
    ret = gtk_dialog_run(dlg->dlg);
    
    if ((*dir)) free((*dir));
    
    if (ret == GTK_RESPONSE_OK) {
	(*dir) = strdup(gtk_entry_get_text(dlg->dirname));
    }
    
    gtk_widget_hide(GTK_WIDGET(dlg->dlg));
    
    if (ret != GTK_RESPONSE_OK) {
	return 0;    
    }
    
    return 1;
}

void make_new_dir()
{
    char *dir = NULL;
    pftp_server_t ftp = NULL;
    
    if (curlist) {
	ftp = getListViewFTP(curlist, 0);
	if (!ftp && getListViewName(curlist))
	    return;
    } else
	return;
    
    if (doNewDirDlg(newdirdlg, &dir)) {
	trim(dir);
	if (dir[0]) {
	    if (ftp) {
		if (!pftp_mkdir(ftp, dir)) {
		    pftp_clear_dir_cache(ftp, ".");
		    pftp_cd(ftp, dir, NULL);
		}
	    } else {
		if (!mkdir(dir, 0755))
		    chdir(dir);
		else
		    messageBox(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
			       GTK_BUTTONS_OK, "Error: %s", strerror(errno));
	    }
	    refreshListView(curlist);
	}
    }
    
    if (dir) free(dir);
}
