/* -*- pftp-c -*- */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <inttypes.h>
#include <pftputil.h>
#include <pftp_client.h>
#include <string.h>
#include <unistd.h>
#include "overwrite_dlg.h"
#include "support.h"

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

static GtkDialog *overwritedlg = NULL;

void initOverwriteDlg(GtkDialog *overwrite, GtkWidget *parent)
{
    overwritedlg = overwrite;
    gtk_window_set_transient_for(GTK_WINDOW(overwritedlg), GTK_WINDOW(parent));
}

int overwrite_dlg(const pftp_file_t existing, const pftp_item_t transfering)
{
    int ret;
    GtkLabel *label;
    size_t len = strlen(existing.name);
    char *tmpstr = malloc(50 + len);
    
    label = GTK_LABEL(lookup_widget(GTK_WIDGET(overwritedlg), "target_file"));
    snprintf(tmpstr, 50 + len, "Target file %s already exists.", 
	     existing.name);
    gtk_label_set_text(label, tmpstr);
    label = GTK_LABEL(lookup_widget(GTK_WIDGET(overwritedlg), "target_size"));
    snprintf(tmpstr, 50 + len, "Target filesize is: %llu bytes.", 
	     existing.size);
    gtk_label_set_text(label, tmpstr);

    len = strlen(transfering->filename);
    tmpstr = realloc(tmpstr, 50 + len);
    
    label = GTK_LABEL(lookup_widget(GTK_WIDGET(overwritedlg), "source_file"));
    snprintf(tmpstr, 50 + len, "Source file %s.", transfering->filename);
    gtk_label_set_text(label, tmpstr);
    label = GTK_LABEL(lookup_widget(GTK_WIDGET(overwritedlg), "source_size"));
    snprintf(tmpstr, 50 + len, "Source filesize is: %llu bytes.", 
	     transfering->size);
    gtk_label_set_text(label, tmpstr);
    free(tmpstr);
    
    if (existing.size < transfering->size) {
	gtk_widget_show(lookup_widget(GTK_WIDGET(overwritedlg), "resume_btn"));
	gtk_widget_show(lookup_widget(GTK_WIDGET(overwritedlg), 
				      "resumeall_btn"));
    } else {
	gtk_widget_hide(lookup_widget(GTK_WIDGET(overwritedlg), "resume_btn"));
	gtk_widget_hide(lookup_widget(GTK_WIDGET(overwritedlg), 
				      "resumeall_btn"));
    }
    
    ret = gtk_dialog_run(overwritedlg);
    
    gtk_widget_hide(GTK_WIDGET(overwritedlg));

    switch (ret) {
    case 1: // Overwrite
	return PFTP_DLG_OVERWRITE;
    case 2: // Overwrite all
	return PFTP_DLG_OVERWRITE | PFTP_DLG_ALL;
    case 3: // Resume
	return PFTP_DLG_RESUME;
    case 4: // Resume all
	return PFTP_DLG_RESUME | PFTP_DLG_ALL;
    case 5: // Skip
	return PFTP_DLG_SKIP;
    case 6: // Skip all
	return PFTP_DLG_SKIP | PFTP_DLG_ALL;
    case 7: // Abort all
	return PFTP_DLG_ABORT;
    default:
	return PFTP_DLG_ABORT;
    }
}
