/* -*- pftp-c -*- */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <inttypes.h>
#include <pftputil.h>
#include "support.h"
#include "rawcmd_dlg.h"
#include "globals.h"
#include "str.h"

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

// From main.c
extern rawcmd_dlg_t rawcmddlg;

struct rawcmd_dlg_s {
    GtkDialog *dlg;
    GtkEntry *cmd;
};

rawcmd_dlg_t initRawCmdDlg(GtkWidget *dlg, GtkWidget *parent)
{
    rawcmd_dlg_t ret = malloc(sizeof(struct rawcmd_dlg_s));
    memset(ret, 0, sizeof(struct rawcmd_dlg_s));
    
    ret->dlg = GTK_DIALOG(dlg);
    
    gtk_window_set_transient_for(GTK_WINDOW(ret->dlg), GTK_WINDOW(parent));
    gtk_dialog_set_default_response(ret->dlg, GTK_RESPONSE_OK);
    
    ret->cmd = GTK_ENTRY(lookup_widget(dlg, "edit_cmd"));
    
    return ret;
}

void freeRawCmdDlg(rawcmd_dlg_t dlg)
{
    if (dlg) {
	free(dlg);
    }
}

static int doRawCmdDlg(rawcmd_dlg_t dlg, char **cmd)
{
    int ret;
    
    ret = gtk_dialog_run(dlg->dlg);
    
    if ((*cmd)) free((*cmd));
    
    if (ret == GTK_RESPONSE_OK) {
	(*cmd) = strdup(gtk_entry_get_text(dlg->cmd));
    }
    
    gtk_widget_hide(GTK_WIDGET(dlg->dlg));
    
    if (ret != GTK_RESPONSE_OK) {
	return 0;    
    }
    
    return 1;
}

void run_rawcmd(int site)
{
    char *cmd = NULL;
    pftp_server_t ftp = NULL;
    
    if (curlist)
	ftp = getListViewFTP(curlist, 0);

    if (!ftp) {	
	return;
    } 
    
    if (doRawCmdDlg(rawcmddlg, &cmd)) {
	trim(cmd);
	if (cmd[0]) {
	    if (site) {
		size_t len = strlen(cmd);
		char *sitecmd = malloc(len + 6);
		memcpy(sitecmd, "SITE ", 5);
		memcpy(sitecmd + 5, cmd, len + 1);
		free(cmd);
		cmd = sitecmd;
	    }
	    
	    pftp_rawcmd(ftp, cmd, NULL);
	}
    }
    
    if (cmd) free(cmd);
}
